/*====================================+=====================================+
! File app_info.h                     ! Copyright (C) 2016-2020 Remi PASCAL !
+-------------------------------------+-------------------------------------+
! This file is part of matelefree.                                          !
! It is free software: you can redistribute it and/or modify it under the   !
! terms of the GNU General Public License as published by the Free Software !
! Foundation, either version 3 of the License, or any later version.        !
! This application is distributed in the hope that it will be useful, but   !
! WITHOUT ANY WARRANTY; without even the implied warranty of                !
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General !
! Public License for more details.                                          !
! You should have received a copy of the GNU General Public License along   !
! with this application. If not, see <http://www.gnu.org/licenses/>.        !
+---------+-----------------------------------------------------------------+
! History !                                                                 !
+---------+                                                                 !
! Date        Version  Description                                          !
! ----------  -------  ---------------------------------------------------- !
! 2020/01/05  1.33     released (build 721)                                 !
! 2019/12/01  1.32     released (build 716)                                 !
! 2019/11/17  1.31     released (build 710) (libs, crash)                   !
! 2019/10/13  1.30     released (build 698) (Pb yt api, twitch api)         !
! 2019/06/30  1.29     released (build 674) (sins)                          !
! 2019/02/17  1.28     released (build 640) (tf1)                           !
! 2019/01/12  1.27     released (build 636) (yt, us)                        !
! 2019/01/09  1.26     released (build 633) (https freebox)                 !
! 2019/01/06  1.25     released (build 632) (tkbar, ren, https, libs)       !
! 2018/11/19  1.24     released (build 620) (Bug bfm, chk upd, etc.)        !
! 2018/11/02  1.23     released (build 617) (Bug chk upd, utf8/8859-1)      !
! 2018/10/14  1.22     released (build 607) (onair)                         !
! 2018/08/09  1.21     released (build 571) (Bug fb playlist)               !
! 2018/08/05  1.20     released (build 569) (TNT except 6play)              !
! 2018/06/29  1.10     released (build 549)                                 !
! 2018/06/10  1.10     beta  1 build 547 released (viewers,periscope,mixer) !
! 2018/03/31  1.01     released (build 507)                                 !
! 2018/03/11  1.00     released (build 504)                                 !
! 2018/03/04  1.00     rc    3 build 498 released                           !
! 2018/02/18  1.00     rc    3 build 493 released                           !
! 2018/01/07  1.00     rc    2 build 480 released                           !
! 2017/12/17  1.00     rc    1 build 477 released (disable sys libs)        !
! 2017/11/22  1.00     beta  2 build 468 released (taskbar icon)            !
! 2017/10/18  1.00     beta  1 build 450 released (mini display mode modif) !
! 2017/10/09  1.00     alpha 5 build 447 released (play crash cancel prefs) !
! 2017/10/08  1.00     alpha 5 build 446 released (ustream vlc init string) !
! 2017/10/07  1.00     alpha 5 build 445 released                           !
! 2017/09/16  1.00     alpha 5 build 433 released                           !
! 2017/07/30  1.00     alpha 4 build 410 released                           !
! 2017/06/04  1.00     alpha 3 build 373 released                           !
! 2017/02/27  1.00     alpha 2 build 298 released                           !
! 2017/02/26  1.00     alpha 2 build 295 released                           !
! 2017/01/08  1.00     alpha 1 build 285 released                           !
! 2017/01/07  1.00     alpha 1 build 284 released                           !
! 2017/01/05  1.00     alpha 1 build 282 released                           !
! 2017/01/03  1.00     alpha 1 build 281 released                           !
! 2017/01/01  1.00     alpha 1 build 278 released                           !
!                                                                           !
+==========================================================================*/



/*-------------------------------------------------------------------------*/
#pragma once
/*-------------------------------------------------------------------------*/
#include "app_build_number.h"
/*-------------------------------------------------------------------------*/



/*-------------------------------------------------------------------------*/
#define APP_NAME                "matelefree"
#define APP_DISPLAY_NAME_ASCII  "MaTeleFree"
#define APP_DISPLAY_NAME_UTF8   u8"MaT\u00e9l\u00e9Free"
#define APP_DISPLAY_NAME        wxString::FromUTF8( APP_DISPLAY_NAME_UTF8 )
#define APP_DISPLAY_NAME_8859   "MaT\Xe9l\Xe9Free"
/*--( Under GTK, "http://" generates 2 url open calls )--------------------*/
#define APP_HOST          "https://scarabee-software.net"
#define APP_URL_SITE      APP_HOST
#define APP_URL_FORUM     APP_HOST "/forum"
#define APP_URL_DONATION  "https://www.paypal.com/cgi-bin/webscr?" \
                            "cmd=_s-xclick&hosted_button_id=BHASPCZZFZU62"
#define APP_URL_VERSION \
        APP_HOST "/download/matelefree/matelefree_version.txt"
/*-------------------------------------------------------------------------*/
#ifdef __x86_64__
   #define APP_CPU_TYPE     "x86_64"
   #define APP_BITNESS_NUM  64
   #define APP_BITNESS_TEXT wxTRANSLATE( "64-bit" )
#elif defined( __i386__ )
   #define APP_CPU_TYPE     "i386"
   #define APP_BITNESS_NUM  32
   #define APP_BITNESS_TEXT wxTRANSLATE( "32-bit" )
#elif defined( __arm__ )
   #define APP_CPU_TYPE     "armhf"
   #define APP_BITNESS_NUM  32
   #define APP_BITNESS_TEXT wxTRANSLATE( "32-bit" )
#else
   #error Unknown platform to: define cpu type and bitness
#endif
/*-------------------------------------------------------------------------*/
#ifdef __WXMSW__
   #define APP_OS_TYPE "windows"
#elif defined( __WXGTK__ )
   #define APP_OS_TYPE "gnulinux"
#else
   #error Unknown platform to: define APP_OS_TYPE
#endif
/*-------------------------------------------------------------------------*/
#define APP_BUILT_TYPE  APP_OS_TYPE "_" APP_CPU_TYPE
/*-------------------------------------------------------------------------*/
#define APP_VERSION       "1.33" //" beta 1 (" APP_BUILD_NUMBER ")"
#define APP_VERSION_MSWRC  1,33,0,0
/*-------------------------------------------------------------------------*/
#define APP_DATES          "2016-2020"
#define APP_COMPANY_ASCII  "Remi PASCAL"
#define APP_COMPANY_UTF8   u8"R\u00e9mi PASCAL"
#define APP_COMPANY        wxString::FromUTF8( APP_COMPANY_UTF8 )
#define APP_COMPANY_8859   "R\Xe9mi PASCAL"
/*-------------------------------------------------------------------------*/
#define APP_COPYRIGHT_ASCII \
        "Copyright (C) " APP_DATES " " APP_COMPANY_ASCII
#define APP_COPYRIGHT_8859 \
        "Copyright \Xa9 " APP_DATES " " APP_COMPANY_8859
#define APP_COPYRIGHT \
          wxString::FromUTF8( u8"Copyright \u00a9" ) \
        + " " APP_DATES " " + APP_COMPANY
/*-------------------------------------------------------------------------*/
#define APP_DESCRIPTION_8859 \
        wxTRANSLATE( "MaT\Xe9l\Xe9\X46ree : a multimedia stream player "\
                     "using VLC's libvlc" \
                   )
#define APP_DESCRIPTION \
        wxString::FromUTF8( wxTRANSLATE( u8"MaT\u00e9l\u00e9Free : " \
                                         "a multimedia stream player " \
                                         "using VLC's libvlc" \
                                       ) \
                          )
/*-------------------------------------------------------------------------*/
#define APP_LICENSE_ASCII \
        wxTRANSLATE( "MaTeleFree is free of charge and " \
                     "distributed under the GPLv3 license" \
                   )
#define APP_LICENSE \
        wxString::FromUTF8( wxTRANSLATE( u8"MaT\u00e9l\u00e9Free is free " \
                                         "of charge and distributed under " \
                                         "the GPLv3 license" \
                                       ) \
                          )

/*--( Under GTK, "http://" generates 2 url open calls )--------------------*/
#define APP_LICENSE_URL "https://www.gnu.org/licenses/gpl.html"

/*-------------------------------------------------------------------------*/
#define APP_MAIL \
        wxString::FromUTF8( u8"MaT\u00e9l\u00e9Free " \
                            "<matelefree@scarabee-software.net>" \
                          )
#define APP_MAIL_TRANSLATOR \
        "English <matelefree.translation@scarabee-software.net>"
/*--------------------------------------------------------------------------+
! IPC communication port for single instance management                     !
! It is only used for the gnu/linux version, on MSW it is DDE               !
+--------------------------------------------------------------------------*/
#define APP_SINS_SERVICE  "19223"
/*-------------------------------------------------------------------------*/



/*==========================================================================+
!                           End of file app_info.h                          !
+==========================================================================*/
