/*====================================+=====================================+
! File CDndChannel.cpp                ! Copyright (C) 2016-2020 Remi PASCAL !
+-------------------------------------+-------------------------------------+
! This file is part of matelefree.                                          !
! It is free software: you can redistribute it and/or modify it under the   !
! terms of the GNU General Public License as published by the Free Software !
! Foundation, either version 3 of the License, or any later version.        !
! This application is distributed in the hope that it will be useful, but   !
! WITHOUT ANY WARRANTY; without even the implied warranty of                !
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General !
! Public License for more details.                                          !
! You should have received a copy of the GNU General Public License along   !
! with this application. If not, see <http://www.gnu.org/licenses/>.        !
+==========================================================================*/



/*-------------------------------------------------------------------------*/
#include <wx/tokenzr.h>
#include <wx/uiaction.h>
#include "CDndChannel.h"
#include "CFrame.h"
/*-------------------------------------------------------------------------*/



/*-------------------------------------------------------------------------*/
void CChannelDataObject::set_text_from_pl_hil( const CPlaylist *pl )
{
   /*----------------------------------------------------------------------*/
   wxString s_text ;
   /*--( Save channels )---------------------------------------------------*/
   for( long l_item = -1 ;
        ( l_item = pl->get_next_item_hil( l_item ) ) != -1 ;
      )
   {  /*-------------------------------------------------------------------*/
      CChannel *p_channel = pl->get_channel( l_item ) ;
      if( p_channel != nullptr )
      {  s_text += p_channel->get_name() + '\n' ; }
      /*-------------------------------------------------------------------*/
   }
   /*----------------------------------------------------------------------*/
   SetText( s_text ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
sc::t_vec_str CChannelDataObject::data_to_vs()
{
   /*----------------------------------------------------------------------*/
   sc::t_vec_str vs_channel ;
   /*----------------------------------------------------------------------*/
   wxString s_text = GetText() ;
   /*--( On MSW "sometimes" a NUL char is added, it has to be removed )----*/
   if( !s_text.empty() && s_text.Last() == '\0' )
   {  s_text.RemoveLast() ; }
   /*----------------------------------------------------------------------*/
   wxStringTokenizer tkz( s_text, "\n" ) ;
   /*----------------------------------------------------------------------*/
   while( tkz.HasMoreTokens() )
   {  vs_channel.emplace_back( tkz.GetNextToken() ) ; }
   /*----------------------------------------------------------------------*/
   return vs_channel ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
wxDragResult CChannelDropTarget::OnData( wxCoord x, wxCoord y,
                                         wxDragResult dr_def
                                       )
{  /*----------------------------------------------------------------------*/
   if( !GetData() ) { return wxDragNone ; }
   /*----------------------------------------------------------------------*/
   CChannelDataObject *p_cdo
                   = static_cast< CChannelDataObject * >( GetDataObject() ) ;
   /*----------------------------------------------------------------------*/
   OnDropChannel( x, y, p_cdo->data_to_vs() ) ;
   /*----------------------------------------------------------------------*/
   return dr_def ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CChannelDropTargetOnPlaylist::OnDropChannel( wxCoord x, wxCoord y,
                                              const sc::t_vec_str &vs_channel
                                                )
{
   /*-----------------------------------------------------------------------+
   ! Search for the item on which the drop is made                          !
   +-----------------------------------------------------------------------*/
   int  i_flags ;
   long l_item_dst = get_win_owner()->HitTest( wxPoint( x, y ), i_flags ) ;
   int  i_part = 0 ;
   /*--( By default they are "appended" to the playlist )------------------*/
   if( l_item_dst == wxNOT_FOUND )
   {  l_item_dst = get_win_owner()->GetItemCount() ; }
   else
   {  /*--( Find out if the drop will be done above or after it )----------*/
      wxRect rect_dst ;
      get_win_owner()->GetItemRect( l_item_dst, rect_dst ) ;
      /*--( Insert before or after this item )-----------------------------*/
      if( y >= rect_dst.y + rect_dst.GetHeight() / 2 )
      {  i_part = 1 ; }
      /*-------------------------------------------------------------------*/
   }

   /*--( The D&D is done between two different playlists )-----------------*/
   if( !get_win_owner()->get_drag_in_progress() )
   {  get_win_owner()->add_channel( vs_channel, l_item_dst + i_part ) ; }
   else
   {
      /*--( The D&D is done in the same window : channels are reordered )--*/
      long l_item_hil = get_win_owner()->get_next_item_hil( -1 ) ;
      /*--( It is useless to D&D a channel on itself )---------------------*/
      if( l_item_hil >= 0 && l_item_hil != l_item_dst )
      {  get_win_owner()->move_channel_hil(   l_item_dst
                                            - l_item_hil
                                            + i_part
                                            + ( l_item_hil < l_item_dst
                                                ? -1 : 0
                                              )
                                          ) ;
      }
      /*-------------------------------------------------------------------*/
   }
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
wxDragResult CChannelDropTargetOnPlaylist::OnDragOver( wxCoord x, wxCoord y,
                                                       wxDragResult dr_def
                                                     )
{
   /*-----------------------------------------------------------------------+
   ! Scroll up / down based on the drag position from top / bottom of list  !
   +-----------------------------------------------------------------------*/
   int  i_win_height   = get_win_owner()->GetClientSize().y ;
   int  i_height_start = get_win_owner()->GetItemSpacing().y / 2 ;
   long l_item_top     = get_win_owner()->GetTopItem() ;
   long l_item_visible = -1 ;
   int  i_pointer_distance =  y ;

   /*--( Scroll up ? )-----------------------------------------------------*/
   if( y < i_height_start )
   {  /*-------------------------------------------------------------------*/
      if( l_item_top > 0 )
      {  l_item_visible = l_item_top - 1 ;
         i_pointer_distance = y ;
      }
      /*-------------------------------------------------------------------*/
   }
   else /*--( Scroll down ? )----------------------------------------------*/
   if( i_win_height - y < i_height_start )
   {  /*-------------------------------------------------------------------*/
      int i_item_view = l_item_top + get_win_owner()->GetCountPerPage() ;
      if( i_item_view < get_win_owner()->GetItemCount() )
      {  l_item_visible = i_item_view ;
         i_pointer_distance = i_win_height - y ;
      }
      /*-------------------------------------------------------------------*/
   }

   /*--( We have to scroll up or down ? )----------------------------------*/
   if( l_item_visible >= 0 )
   {
      /*-------------------------------------------------------------------*/
      get_win_owner()->set_item_visible( l_item_visible ) ;
      /*--( Closer the mouse is from top/bottom, higher the speed is )-----*/
      if( i_height_start > 0 )
      {  /*----------------------------------------------------------------*/
         const int co_i_sleep_max = 200 ;
         int i_sleep = i_pointer_distance * co_i_sleep_max / i_height_start ;
         if( i_sleep > 0 ) { wxMilliSleep( i_sleep ) ; }
         /*----------------------------------------------------------------*/
      }
      /*-------------------------------------------------------------------*/
   }

   /*-----------------------------------------------------------------------+
   ! The drop is ok on playlists accepting channels or the ones accepting   !
   ! them to be reordered                                                   !
   +-----------------------------------------------------------------------*/
   return      get_win_owner()->get_accept_ch()
            || (    get_win_owner()->get_order_ch()
                 && get_win_owner()->get_drag_in_progress()
               )
          ? dr_def : wxDragNone ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CChannelDropTargetOnTree::OnDropChannel( wxCoord x, wxCoord y,
                                              const sc::t_vec_str &vs_channel
                                            )
{
   /*----------------------------------------------------------------------*/
   CBookPlaylist *bopl = wxGetApp().m_frame->get_bopl() ;
   wxTreeItemId ti = bopl->get_hittest( wxPoint( x, y ) ) ;
   if( !ti.IsOk() ) { return ; }
   /*--( The playlist has to accept the operation )------------------------*/
   CPlaylist *pl = bopl->get_playlist( ti ) ;
   if( pl == nullptr || !pl->get_accept_ch() ) { return ; }
   /*----------------------------------------------------------------------*/
   pl->add_channel( vs_channel ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
wxDragResult CChannelDropTargetOnTree::OnDragOver( wxCoord x, wxCoord y,
                                                   wxDragResult dr_def
                                                 )
{
   /*----------------------------------------------------------------------*/
   CBookPlaylist *bopl = wxGetApp().m_frame->get_bopl() ;
   int i_ht_flags ;
   wxTreeItemId ti = bopl->get_hittest( wxPoint( x, y ), i_ht_flags ) ;
   if( !ti.IsOk() ) { return wxDragNone ; }

   /*--( Select this playlist if not the current one )---------------------*/
   if( ti != bopl->get_selection() ) { bopl->set_selection( ti ) ; }

   /*--( Open tree node when the cursor is on the '+' )--------------------*/
   if(    ( i_ht_flags & wxTREE_HITTEST_ONITEMBUTTON ) != 0
       &&  bopl->HasChildren( ti )
       && !bopl->IsExpanded( ti )
     )
   {  bopl->Expand( ti ) ;
      return wxDragNone ;
   }

   /*-----------------------------------------------------------------------+
   ! Scroll up / down based on the drag position from top / bottom of list  !
   +-----------------------------------------------------------------------*/
   wxRect rect_item ;
   bopl->GetBoundingRect( ti, rect_item ) ;
   int i_win_height   = bopl->GetClientSize().y ;
   int i_height_start = rect_item.GetHeight() / 2 ;
   int i_pointer_distance = -1 ;

   /*--( Scroll up ? )-----------------------------------------------------*/
   if( y < i_height_start )
   {
      /*--( Search for a brother "up" )------------------------------------*/
      wxTreeItemId ti_prev = bopl->GetPrevSibling( ti ) ;
      if( ti_prev.IsOk() )
      {  /*----------------------------------------------------------------*/
         ti = ti_prev ;
         /*--( What is up is in fact the last family child of item found )-*/
         for( ; bopl->IsExpanded( ti ) && bopl->HasChildren( ti ) ;
              ti = bopl->GetLastChild( ti )
            )
         {  ; }
         /*----------------------------------------------------------------*/
      }
      else
      {  /*--( No brother ? we go up except for root )---------------------*/
         ti_prev = bopl->GetItemParent( ti ) ;
         if( ti_prev != bopl->GetRootItem() )
         {  ti = ti_prev ; }
         /*----------------------------------------------------------------*/
      }
      /*-------------------------------------------------------------------*/
      if( ti.IsOk() )
      {  /*----------------------------------------------------------------*/
         bopl->EnsureVisible( ti ) ;
         bopl->set_selection( ti ) ;
         i_pointer_distance = y ;
         /*----------------------------------------------------------------*/
      }
      /*-------------------------------------------------------------------*/
   }
   else /*--( Scroll down ? )----------------------------------------------*/
   if( i_win_height - y < i_height_start )
   {
      /*--( Search for child if visible )----------------------------------*/
      if( bopl->IsExpanded( ti ) && bopl->HasChildren( ti ) )
      {  wxTreeItemIdValue cookie ;
         ti = bopl->GetFirstChild( ti, cookie ) ;
      }
      else /*--( No children ? Search for a brother "down" )---------------*/
      {
         wxTreeItemId ti_next = bopl->GetNextSibling( ti ) ;
         /*-----------------------------------------------------------------+
         ! No brother (last child) ? It means we search for the first       !
         ! child down the parent line                                       !
         +-----------------------------------------------------------------*/
         if( !ti_next.IsOk() )
         {  /*-------------------------------------------------------------*/
            wxTreeItemId ti_parent = ti ;
            /*-------------------------------------------------------------*/
            while( ti_parent.IsOk() && !ti_next.IsOk() )
            {  /*----------------------------------------------------------*/
               ti_parent = bopl->GetItemParent( ti_parent ) ;
               /*--( Under MSW nextsibling of root crashes/asserts )-------*/
               if( ti_parent.IsOk() && ti_parent != bopl->GetRootItem() )
               {  ti_next = bopl->GetNextSibling( ti_parent ) ; }
               /*----------------------------------------------------------*/
            }
            /*-------------------------------------------------------------*/
         }
         /*----------------------------------------------------------------*/
         if( ti_next.IsOk() ) { ti = ti_next ; }
         /*----------------------------------------------------------------*/
      }
      /*-------------------------------------------------------------------*/
      if( ti.IsOk() )
      {  /*----------------------------------------------------------------*/
         bopl->EnsureVisible( ti ) ;
         bopl->set_selection( ti ) ;
         i_pointer_distance = i_win_height - y ;
         /*----------------------------------------------------------------*/
      }
      /*-------------------------------------------------------------------*/
   }

   /*--( Closer the mouse is from top/bottom, higher the speed is )--------*/
   if( i_pointer_distance > 0 && i_height_start > 0 )
   {  /*-------------------------------------------------------------------*/
      const int co_i_sleep_max = 500 ;
      int i_sleep = i_pointer_distance * co_i_sleep_max / i_height_start ;
      if( i_sleep > 0 ) { wxMilliSleep( i_sleep ) ; }
      /*-------------------------------------------------------------------*/
   }

   /*--( The playlist has to accept the operation )------------------------*/
   CPlaylist *pl = bopl->get_playlist( ti ) ;
   if( pl == nullptr || !pl->get_accept_ch() ) { return wxDragNone ; }
   /*----------------------------------------------------------------------*/
   return dr_def ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/



/*==========================================================================+
!                         End of file CDndChannel.cpp                       !
+==========================================================================*/
