/*====================================+=====================================+
! File CPlSubWin.cpp                  ! Copyright (C) 2016-2020 Remi PASCAL !
+-------------------------------------+-------------------------------------+
! This file is part of matelefree.                                          !
! It is free software: you can redistribute it and/or modify it under the   !
! terms of the GNU General Public License as published by the Free Software !
! Foundation, either version 3 of the License, or any later version.        !
! This application is distributed in the hope that it will be useful, but   !
! WITHOUT ANY WARRANTY; without even the implied warranty of                !
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General !
! Public License for more details.                                          !
! You should have received a copy of the GNU General Public License along   !
! with this application. If not, see <http://www.gnu.org/licenses/>.        !
+==========================================================================*/



/*-------------------------------------------------------------------------*/
#include "wx/uri.h"
#include "wx/socket.h"
#include "CFrame.h"
#include "CDlgAddLive.h"
#include "CPlSubWin.h"
/*-------------------------------------------------------------------------*/



/*-------------------------------------------------------------------------*/
CPlSubWin::CPlSubWin( wxWindow *parent, CPlaylist *pl )
         : wxPanel( parent ), m_pl( pl )
{
   /*----------------------------------------------------------------------*/
   m_bt_start = new wxButton( this, wxID_ANY, wxEmptyString ) ;
   m_ga_progress = new wxGauge( this, wxID_ANY, 1, wxDefaultPosition,
                                wxSize( -1, 7 )
                              ) ;
     m_stx_msg
   = new wxStaticText( this, wxID_ANY, wxEmptyString,
                       wxDefaultPosition, wxDefaultSize,
                       wxALIGN_CENTRE_HORIZONTAL | wxST_NO_AUTORESIZE
                     ) ;
   m_ga_progress->SetValue( 0 ) ;
   /*--( Information will be displayed once treatment started )------------*/
   m_ga_progress->Show( false ) ;
   m_stx_msg->Show( false ) ;
   /*----------------------------------------------------------------------*/
   m_bsz = new wxBoxSizer( wxVERTICAL ) ;
   m_bsz->Add( m_bt_start, 0, wxALIGN_CENTER, 5 ) ;
   m_bsz->Add( m_ga_progress, 1, wxEXPAND | wxTOP | wxBOTTOM, 5 ) ;
   m_bsz->Add( m_stx_msg, 0, wxEXPAND, 5 ) ;
   /*----------------------------------------------------------------------*/
   m_bt_start->Bind( wxEVT_BUTTON,
                     [ this ]( wxCommandEvent &event ) { start() ; }
                   ) ;
   /*----------------------------------------------------------------------*/
   SetSizer( m_bsz ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CPlSubWin::reset_gui_search()
{
   /*----------------------------------------------------------------------*/
   enable_start( true ) ;
   m_pl->SetFocus() ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CPlSubWin::show_info()
{
   /*----------------------------------------------------------------------*/
   m_ga_progress->Show( true ) ;
   m_stx_msg->Show( true ) ;
   m_pl->get_main_window()->Layout() ;
   m_pl->get_main_window()->Update() ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CPlSubWin::msg_info( const wxString &s )
{
   /*----------------------------------------------------------------------*/
   m_stx_msg->SetForegroundColour( *wxBLUE ) ;
   m_stx_msg->SetLabelText( s ) ;
   m_stx_msg->Update() ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CPlSubWin::msg_channel_found( int i_nb_found )
{
   /*----------------------------------------------------------------------*/
   msg_info(   wxString::Format( "%d/%d ",
                                 i_nb_found, m_ga_progress->GetRange()
                               )
             + wxPLURAL( "channel found", "channels found", i_nb_found )
           ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CPlSubWin::msg_error( const wxString &s )
{
   /*----------------------------------------------------------------------*/
   m_stx_msg->SetForegroundColour( *wxRED ) ;
   m_stx_msg->SetLabelText( s ) ;
   m_stx_msg->Update() ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
CPlSubWinFbtvoOk::CPlSubWinFbtvoOk( wxWindow *parent, CPlaylist *pl_fbtvo_ok)
                : CPlSubWin( parent, pl_fbtvo_ok )
{
   /*----------------------------------------------------------------------*/
   m_bt_start->SetLabel( _( "Search" ) ) ;
   m_bt_start->SetToolTip( _( "Search for \"En option\" channels that "
                              "you can currently watch"
                            )
                         ) ;
   /*----------------------------------------------------------------------*/
   msg_info( _( "\"En option\" accessible channels" ) ) ;

   /*--( For this treatment information is always displayed )--------------*/
   show_info() ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
static bool st_create_freebox_rtsp_socket( const wxURI &uri,
                                           wxSocketClient &socket
                                         )
{
   /*----------------------------------------------------------------------*/
   long l_port ;
   /*----------------------------------------------------------------------*/
   if( socket.IsOk() && socket.IsConnected() )
   {  socket.Close() ; }
   /*----------------------------------------------------------------------*/
   if( uri.GetScheme() != "rtsp" )
   {  return false ; }
   /*----------------------------------------------------------------------*/
   if( uri.GetPort().empty() || !uri.GetPort().ToLong( &l_port ) )
   {  l_port = 554 ; }
   /*----------------------------------------------------------------------*/
   wxIPV4address addr ;
   if( !addr.Hostname( uri.GetServer() ) || !addr.Service( l_port ) )
   {  return false ; }
   /*----------------------------------------------------------------------*/
   socket.Connect( addr, false ) ;
   if( !socket.IsConnected() && !socket.WaitOnConnect( 5, 0 ) )
   {  return false ; }
   /*----------------------------------------------------------------------*/
   return true ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
static bool st_check_rtsp_stream( wxSocketClient &socket,
                                  const wxString &s_url
                                )
{
   /*----------------------------------------------------------------------*/
   char     a_c_buffer[ 1024 ] ;
   wxUint32 dw_length ;
   /*----------------------------------------------------------------------*/
     dw_length
   = sprintf( a_c_buffer,
              "SETUP %s RTSP/1.0\r\n"
              "CSeq: 1\r\n"
              "Transport: RTP/AVP;unicast;client_port=9000-9001\r\n"
              "\r\n",
              s_url.mb_str().data()
            ) ;
   /*--( Write command : no multiple write are expected )------------------*/
   socket.Write( a_c_buffer, dw_length ) ;
   if( socket.Error() || socket.LastWriteCount() != dw_length )
   {  return false ; }
   /*--( Read answer )-----------------------------------------------------*/
   socket.Read( a_c_buffer, sizeof( a_c_buffer ) ) ;
   if( socket.Error() || ( dw_length = socket.LastReadCount() ) < 20 )
   {  return false ; }

   /*--( If a session number is present the channel is accessible )--------*/
   char *p_c_session_start = strstr( a_c_buffer, "\nSession: " ) ;
   if( p_c_session_start == nullptr ) { return false ; }

   /*-----------------------------------------------------------------------+
   ! Now we have got to clean up this started session                       !
   +-----------------------------------------------------------------------*/

   /*--( Search for the end of the line after skipping the "\n" )----------*/
   char *p_c_session_end = strpbrk( ++p_c_session_start, "\r\n" ) ;
   if( p_c_session_end == nullptr ) { return false ; }
   /*----------------------------------------------------------------------*/
     dw_length
   = sprintf( a_c_buffer,
              "TEARDOWN %s RTSP/1.0\r\n"
              "CSeq: 2\r\n"
              "%.*s\r\n"
              "\r\n",
              s_url.mb_str().data(),
              static_cast< int >( p_c_session_end - p_c_session_start ),
              p_c_session_start
            ) ;
   /*--( Send the command )------------------------------------------------*/
   socket.Write( a_c_buffer, dw_length ) ;
   if( socket.Error() || socket.LastWriteCount() != dw_length )
   {  return false ; }
   /*--( Clean the rest )--------------------------------------------------*/
   socket.Read( a_c_buffer, sizeof( a_c_buffer ) ) ;
   if( socket.Error() ) { return false ; }

   /*--( If we are here this means we have a positive result )-------------*/
   return true ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CPlSubWinFbtvoOk::start()
{
   /*--( The declarations are done here because of the "goto" )------------*/
   const CPlaylist *pl_fbtvo = wxGetApp().get_pk_pl( PK_FBTVO ) ;
   wxSocketClient socket ;
   sc::t_vec_str vs_found ;
   sc::t_vec_str vs_add ;
   wxURI uri ;
   wxURI uri_prev ;
   int i_nb_same = 0 ;

   /*----------------------------------------------------------------------*/
   const size_t co_sz_nb_option
          = ( pl_fbtvo == nullptr ? 0 : pl_fbtvo->get_vs_channel().size() ) ;
   /*----------------------------------------------------------------------*/
   if( co_sz_nb_option == 0 )
   {  msg_error( _( "No \"en option\" channel available" ) ) ;
      return ;
   }
   /*----------------------------------------------------------------------*/
   enable_start( false ) ;
   m_ga_progress->SetRange( co_sz_nb_option ) ;
   msg_info( _( "Searching ..." ) ) ;
   /*----------------------------------------------------------------------*/
   for( size_t sz_pos = 0 ; sz_pos < co_sz_nb_option ; ++sz_pos )
   {
      /*-------------------------------------------------------------------*/
      CChannel *p_channel = pl_fbtvo->get_channel( sz_pos ) ;
      const str_stream *p_stream ;
      if(    p_channel != nullptr
          && ( p_stream = p_channel->get_stream() ) != nullptr
        )
      {
         /*--( Extract server to create the socket )-----------------------*/
         uri.Create( p_stream->m_s_url ) ;
         /*--( The socket is recreated only when needed )------------------*/
         if(    uri.GetScheme() != uri_prev.GetScheme()
             || uri.GetServer() != uri_prev.GetServer()
             || uri.GetPort()   != uri_prev.GetPort()
           )
         {  /*-------------------------------------------------------------*/
            if( !st_create_freebox_rtsp_socket( uri, socket ) )
            {  msg_error( _( "Connection problem with the Freebox" ) ) ;
               reset_gui_search() ;
               return ;
            }
            /*-------------------------------------------------------------*/
            uri_prev = uri ;
            /*-------------------------------------------------------------*/
         }
         /*----------------------------------------------------------------*/
         if( st_check_rtsp_stream( socket, p_stream->m_s_url ) )
         {
            /*-------------------------------------------------------------*/
            vs_found.emplace_back( p_channel->get_name() ) ;
            if( m_pl->get_channel_item( p_channel->get_name() ) >= 0 )
            {  ++i_nb_same ; }
            else
            {  vs_add.emplace_back( p_channel->get_name() ) ; }
            /*-------------------------------------------------------------*/
            msg_channel_found( static_cast< int >( vs_found.size() ) ) ;
            /*-------------------------------------------------------------*/
         }
         /*----------------------------------------------------------------*/
      }
      /*-------------------------------------------------------------------*/
      m_ga_progress->SetValue( sz_pos + 1 ) ;
      /*-------------------------------------------------------------------*/
   }

   /*----------------------------------------------------------------------*/
   int i_nb_add = 0 ;
   int i_nb_del = 0 ;
   wxString s_msg_add ;
   wxString s_msg_del ;
   /*----------------------------------------------------------------------*/
   i_nb_add = static_cast< int >( vs_add.size() ) ;
   if( m_pl->get_nb_item() > 0 )
   {  i_nb_del = m_pl->get_nb_item() - i_nb_same ; }
   /*----------------------------------------------------------------------*/
   if( i_nb_add != 0 )
   {  s_msg_add.Printf( wxPLURAL( "%d channel added",
                                  "%d channels added", i_nb_add
                                ),
                        i_nb_add
                      ) ;
      if( i_nb_del != 0 ) { s_msg_add += ", " ; }
   }
   /*----------------------------------------------------------------------*/
   if( i_nb_del != 0 )
   {  s_msg_del.Printf( wxPLURAL( "%d channel deleted",
                                  "%d channels deleted", i_nb_del
                                ),
                        i_nb_del
                      ) ;
   }
   /*----------------------------------------------------------------------*/
   if( s_msg_add.empty() && s_msg_del.empty() )
   {  msg_info( _( "No change found" ) ) ; }
   else
   {  msg_info( s_msg_add + s_msg_del ) ; }

   /*--( Finally add the channels found )----------------------------------*/
   m_pl->clear() ;
   if( !vs_found.empty() )
   {  /*-------------------------------------------------------------------*/
      m_pl->add_channel( vs_found ) ;
      /*--( The new channels are highlighted )-----------------------------*/
      if( !vs_add.empty() )
      {  /*----------------------------------------------------------------*/
         m_pl->set_item_hil( -1, false ) ;
         m_pl->set_item_hil( vs_add, true ) ;
         /*--( The focus is set to the 1st hil to avoid confusion )--------*/
         long l_item = m_pl->get_last_item_hil() ;
         if( l_item >= 0 )
         {  /*-------------------------------------------------------------*/
            m_pl->set_item_visible( l_item ) ;
            l_item = m_pl->get_first_item_hil() ;
            m_pl->set_item_focus( l_item ) ;
            m_pl->set_item_visible( l_item ) ;
            /*-------------------------------------------------------------*/
         }
         /*----------------------------------------------------------------*/
      }
      /*-------------------------------------------------------------------*/
   }
   /*----------------------------------------------------------------------*/
   reset_gui_search() ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
CPlSubWinChOnair::CPlSubWinChOnair( wxWindow *parent, CPlaylist *pl,
                                    CPlSubWinLive *subwinlive
                                  )
                : CPlSubWin( parent, pl ), m_subwinlive( subwinlive )

{
   /*----------------------------------------------------------------------*/
   m_bt_start->SetLabel( _( "On air ?" ) ) ;
   m_bt_start->SetToolTip( _( "Search for channels in this playlist that "
                              "are currently on air"
                            )
                         ) ;
   /*----------------------------------------------------------------------*/
   Bind( EVT_COA_INFO,
         [ this ]( CCOAInfoEvent &event )
         {  run_info( event.get_item(), event.get_nb_found() ) ; }
       ) ;
   Bind( EVT_COA_END,
         [ this ]( sc::CEvent &event )
         {  finish() ; }
       ) ;
   /*----------------------------------------------------------------------*/
}

/*--( Check "on air" messages )--------------------------------------------*/
wxDEFINE_EVENT( EVT_COA_INFO, CCOAInfoEvent ) ;
wxDEFINE_EVENT( EVT_COA_END,  sc::CEvent ) ;

/*-------------------------------------------------------------------------*/
class CThreadRefreshOnair : public wxThread
{
   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      explicit CThreadRefreshOnair( CPlaylist *pl ) : m_pl( pl )
      {  ; }
      //~CThreadRefreshOnair() ;

   /*----------------------------------------------------------------------*/
   private :
      /*-------------------------------------------------------------------*/
      CPlaylist *m_pl ;

   /*----------------------------------------------------------------------*/
   public :
      /*--( Will be executed by "Run" )------------------------------------*/
      virtual wxThread::ExitCode Entry() override
      {
         /*----------------------------------------------------------------*/
         const int co_i_nb_channel
                 = static_cast< int >( m_pl->get_vs_channel().size() ) ;
         /*----------------------------------------------------------------*/
         CPlSubWinChOnair *win = m_pl->get_win_onair() ;
         if( win == nullptr ) { return ( wxThread::ExitCode )-1 ; }
         /*----------------------------------------------------------------*/
         int i_nb_found = 0 ;
         /*----------------------------------------------------------------*/
         for( int i_item = 0 ; i_item < co_i_nb_channel ; ++i_item )
         {
            /*-------------------------------------------------------------*/
            CChannel *p_channel = m_pl->get_channel( i_item ) ;
            /*-------------------------------------------------------------*/
            if(    p_channel != nullptr
                && p_channel->get_use_attr_onair()
                && p_channel->refresh_onair() == 0
                && p_channel->get_onair()
              )
            {  ++i_nb_found ; }
            /*-------------------------------------------------------------*/
            wxQueueEvent( win, new CCOAInfoEvent( i_item, i_nb_found ) ) ;
            /*-------------------------------------------------------------*/
         }
         /*----------------------------------------------------------------*/
         wxQueueEvent( win, new sc::CEvent( EVT_COA_END ) ) ;
         /*----------------------------------------------------------------*/
         return ( wxThread::ExitCode )0 ;
         /*----------------------------------------------------------------*/
      }

   /*----------------------------------------------------------------------*/
} ;

/*-------------------------------------------------------------------------*/
void CPlSubWinChOnair::start()
{
   /*--( Information is now displayed )------------------------------------*/
   show_info() ;

   /*--( Change proportions of subwindows to make space for messages )-----*/
   if( m_subwinlive != nullptr )
   {  m_subwinlive->expand_win_onair() ; }
   /*----------------------------------------------------------------------*/
   const size_t co_sz_nb_channel = m_pl->get_vs_channel().size() ;
   /*----------------------------------------------------------------------*/
   if( co_sz_nb_channel == 0 )
   {  msg_error( _( "No channel available" ) ) ;
      return ;
   }
   /*----------------------------------------------------------------------*/
   enable_start( false ) ;
   m_ga_progress->SetValue( 0 ) ;
   m_ga_progress->SetRange( co_sz_nb_channel ) ;
   msg_info( _( "Searching ..." ) ) ;

   /*--( The "real" run )--------------------------------------------------*/
   CThreadRefreshOnair *p_thread = new CThreadRefreshOnair( m_pl ) ;
   p_thread->Run() ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CPlSubWinChOnair::run_info( int i_item, int i_nb_found )
{
   /*--( The item may become "off the air" and so has to be refreshed )----*/
   m_pl->refresh_item( i_item ) ;
   m_ga_progress->SetValue( i_item + 1 ) ;
   /*----------------------------------------------------------------------*/
   if( i_nb_found <= 0 )
   {  msg_info( _( "No channel found" ) ) ; }
   else
   {  msg_channel_found( i_nb_found ) ; }
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CPlSubWinChOnair::finish()
{
   /*----------------------------------------------------------------------*/
   reset_gui_search() ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
CPlSubWinLive::CPlSubWinLive( wxWindow *parent, CPlaylistLive *pl_live )
             : wxPanel( parent ), m_pl( pl_live )
{
   /*----------------------------------------------------------------------*/
   m_bt_add = new wxButton( this, wxID_ANY, _( "Add" ) ) ;
   m_bt_add->SetToolTip( _( "Add new channels to this playlist" ) ) ;
   m_win_onair = new CPlSubWinChOnair( this, m_pl, this ) ;
   /*----------------------------------------------------------------------*/
   m_bsz = new wxBoxSizer( wxHORIZONTAL ) ;
   /*--( The proportions will be changed after start has been clicked )----*/
   m_bsz->AddStretchSpacer( 1 ) ;
   m_bsz->Add( m_bt_add, 0, wxALL | wxALIGN_TOP, 5 ) ;
   m_bsz->AddStretchSpacer( 1 ) ;
   m_bsz->Add( m_win_onair, 0, wxALL | wxEXPAND, 5 ) ;
   m_bsz->AddStretchSpacer( 1 ) ;
   SetSizer( m_bsz ) ;
   /*----------------------------------------------------------------------*/
   m_bt_add->Bind( wxEVT_BUTTON,
                   [ this ]( wxCommandEvent &event )
                   {  CDlgAddLive( wxGetApp().m_frame, m_pl ).get_modal() ; }
                 ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CPlSubWinLive::expand_win_onair()
{
   /*-----------------------------------------------------------------------+
   ! If not already done, change proportions of subwindows to make space    !
   ! for messages                                                           !
   +-----------------------------------------------------------------------*/
   wxSizer *bsz = GetSizer() ;
   if( bsz->GetItem( static_cast< size_t >( 0 ) )->GetProportion() != 0 )
   {  /*-------------------------------------------------------------------*/
      bsz->GetItem( static_cast< size_t >( 0 ) )->SetProportion( 0 ) ;
      bsz->GetItem( static_cast< size_t >( 2 ) )->SetProportion( 0 ) ;
      bsz->GetItem( static_cast< size_t >( 4 ) )->SetProportion( 0 ) ;
      bsz->GetItem( get_win_onair() )->SetProportion( 1 ) ;
      get_pl()->get_main_window()->Layout() ;
      get_pl()->get_main_window()->Update() ;
      /*-------------------------------------------------------------------*/
   }
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/



/*==========================================================================+
!                       End of file CPlSubWin.cpp                           !
+==========================================================================*/
